<?php

namespace D3\SensitiveMessageFormatter;

use GuzzleHttp\MessageFormatter;
use Psr\Http\Message\RequestInterface;
use Psr\Http\Message\ResponseInterface;
use Throwable;

class sensitiveMessageFormatter extends MessageFormatter
{
    public function __construct(
        ?string $template = self::CLF,
        protected array $anonymizations = [],
        protected ?string $replaceChar = null
    ) {
        $this->createReplacements($this->anonymizations);

        parent::__construct($template);
    }

    protected function createReplacements(array $search = []): void
    {
        $replacements = [];
        $this->replaceChar = $this->replaceChar ?? '*';

        array_map(
            function ($search) use (&$replacements) {
                $replacements[$search] = str_repeat($this->replaceChar, strlen($search));
                $replacements[urlencode($search)] = str_repeat($this->replaceChar, strlen($search));
            },
            $search
        );

        $this->anonymizations = $replacements;
    }

    public function format(
        RequestInterface $request,
        ResponseInterface $response = null,
        Throwable $error = null
    ): string
    {
        $result = parent::format($request, $response, $error);

        if (count($this->anonymizations)) {
            $result = str_replace(
                array_keys($this->anonymizations),
                array_values($this->anonymizations),
                $result
            );
        }

        return $result;
    }
}